/******************************************************************************

AUTOLIV ELECTRONIC document.

-------------------------------------

Copyright Tuerme Inc. All rights reserved.

*******************************************************************************
C-File Template Version:
******************************************************************************/
/* PRQA S 0288 ++ */
/*
 * Explanation:
 *    Source file for s-function wrapper Fusion Core in CADS4 simulink
 */
/*
 */
/* PRQA S 0288 -- */
/*!****************************************************************************

@details
   <Describes details of this module 'Template.c' file within overall
    context of component implementation>

******************************************************************************/
/******************************************************************************
EXTERNAL DEPENDENCIES
******************************************************************************/

#include "sfObjectFusionMpf_func.h"
#include "Fusion_SLAM_types.h"

#include "stddef.h"
#include "assert.h" 

#ifndef TU_ALGO_SIMULATION_VS_CONF_200_DT
#pragma section  ".data_cpu1"
#endif

/******************************************************************************
MODULE DEFINES
******************************************************************************/

/******************************************************************************
MODULE TYPES
******************************************************************************/

/******************************************************************************
DECLARATION OF LOCAL FUNCTIONS
******************************************************************************/

/******************************************************************************
DEFINITION OF EXPORTED VARIABLES
******************************************************************************/

/******************************************************************************
DEFINITION OF LOCAL CONSTANT DATA
******************************************************************************/

/******************************************************************************
DEFINITION OF EXPORTED CONSTANT DATA
******************************************************************************/

/******************************************************************************
MODULE FUNCTION-LIKE MACROS
******************************************************************************/

/******************************************************************************
DEFINITION OF LOCAL FUNCTION
******************************************************************************/

/******************************************************************************
DEFINITION OF APIs
******************************************************************************/

/*!****************************************************************************
* @details
*     Initialize object fusion with input parameters. An object of type TueObjFusn_ParametersType is
*     initialized to hold fusion parameters. Using getters, the value of the parameters are retrieved 
*     throughout the code. 
*     @see TueObjFusn_ParametersType
*     @see TueObjFusn_Params.h
* @param[in]
*     const uint32 u32SensorMode
* @param[in]
*     const uint8  u8FusionMode
* @param[in]
*     const float32 f32PostPrediction
* @param[in]
*     const uint8  u8WeightMode
* @param[in]
*     const float32 f32MatchGate
* @param[in]
*     const uint8 u8OosmMode
* @param[in]
*     const uint8 u8AssociationMode
* @param[in]
*     const boolean bUseForMotionAccX
* @param[in]
*     const boolean bUseForMotionAccY
* @param[out]
*     none
* @return
*     none
******************************************************************************/
void sfObjectFusionMpf_Init_wrapper(const uint32 u32SensorMode,
                                    const float32 f32MatchGate,
                                    const float32 f32PedestrianVarianceInXForQ,
                                    const float32 f32PedestrianVarianceInYForQ,
                                    const float32 f32VehicleVarianceInXForQ,
                                    const float32 f32VehicleVarianceInYForQ,
                                    const boolean bUseTrackMerge,
                                    const float32 f32AdditionalQonDiagonal,
                                    const boolean bOutputIsOverground,
                                    const boolean bUseCoasting,
                                    const uint32 u32CoastedSensor){
   uint32 u32Success = TRUE;

   Fusion_init();
   
   u32Success = Fusion_set_u32SensorMode(u32SensorMode);
   u32Success = Fusion_set_f32MatchGate(f32MatchGate);
   
   u32Success = Fusion_set_f32PedestrianVarianceInXForQ(f32PedestrianVarianceInXForQ);
   u32Success = Fusion_set_f32PedestrianVarianceInYForQ(f32PedestrianVarianceInYForQ);

   u32Success = Fusion_set_f32VehicleVarianceInXForQ(f32VehicleVarianceInXForQ);
   u32Success = Fusion_set_f32VehicleVarianceInYForQ(f32VehicleVarianceInYForQ);

   u32Success = Fusion_set_bUseTrackMerge(bUseTrackMerge);


   u32Success = Fusion_set_bOutputIsOverground(bOutputIsOverground);
   u32Success = Fusion_set_bUseCoasting(bUseCoasting);
//#if ( defined(CAMERA_MINIEYE) && CAMERA_MINIEYE == 1 )
//   OtherCarLocHistoryInit();//Init Other car loaction history for other car velocity calculation

//   //Ticket:http://119.3.139.124:10101/tickets/DFA-ZT19.git/31 changed by changchang 20200603 start
//   PedestrianLocHistoryInit();//Init pedestrian loaction history for pedestrian position and velocity calculation
//  //Ticket:http://119.3.139.124:10101/tickets/DFA-ZT19.git/31 changed by changchang 20200603 end
//#endif
}

/**
 * @brief Fusion core wrapper function
 * 
 * Main wrapper function that wraps around the TUE Fusion core
 * and calls main fusion function. A part of preprocessing 
 * functionality is called here.
 * 
 * @param[in] pLidarObjects Lidar object list
 * @param[in] pCamObjects Camera object list
 * @param[in] EgoMotion Ego vehicle dynamic input list
 * @param[in] CycleDeltaTime Fusion cycle delta time
 * @param[out] pFusnObjLists_Output Fusion result object list
 * @param[out] pErrorBuffer Error buffer list
 * 
 * @return None
******************************************************************************/
void sfObjectFusionMpf_Outputs_wrapper(const TueObjFusn_ObjectListType* const pLidarObjects,
						               const TueObjFusn_ObjectListType* const pCamObjects,
	                                   const TueObjFusn_EgoMotionType* const EgoMotion,
	                                   const float32 CycleDeltaTime,
                                       TueObjFusn_ObjectListType* const pFusnObjLists_Output,
                                       TueObjFusn_ErrorBufferType* const pErrorBuffer)
{
    uint32 u32Success;

    static TueObjFusn_ObjectListType *inObjListVector[3];   /* array of pointers to lists */                /* only internal, unused in Simulink implementation */


    //init camera and lidar object list for the Fusion function
    Trackable_initObjectList(pFusnObjLists_Output);

    //set camera/lidar object list for fusion input array
    if (pCamObjects == NULL){
	    inObjListVector[0] = pLidarObjects;//camera object list is null, use lidar object in case error happed in fusion
    }
    else{
        inObjListVector[0] = pCamObjects;
    }
    inObjListVector[1] = pLidarObjects;
    inObjListVector[2] = (TueObjFusn_ObjectListType*)NULL_PTR; /* the fusion cycle method expects the input list to be NULL terminated (unless n == N) */

    //excute fusion process
    u32Success = PerformFusionCycle(inObjListVector, EgoMotion, CycleDeltaTime, pFusnObjLists_Output,pErrorBuffer);
}

/******************************************************************************
End Of File
*****************************************************************************/
